var openOptions = [
    { propName: "openMode", type: "select", displayText: "OpenMode", options: [{ name: 'normal', value: 0 }, { name: 'lazy', value: 1 }, { name: 'incremental', value: 2 }], default: 0 },
    { propName: "includeStyles", type: "boolean", default: true },
    { propName: "includeFormulas", type: "boolean", default: true },
    { propName: "fullRecalc", type: "boolean", default: false },
    { propName: "dynamicReferences", type: "boolean", default: true },
    { propName: "calcOnDemand", type: "boolean", default: false },
    { propName: "includeUnusedStyles", type: "boolean", default: true },
];

var importXlsxOptions = [
    { propName: "openMode", type: "select", displayText: "OpenMode", options: [{ name: 'normal', value: 0 }, { name: 'lazy', value: 1 }, { name: 'incremental', value: 2 }], default: 0 },
    { propName: "includeStyles", type: "boolean", default: true },
    { propName: "includeFormulas", type: "boolean", default: true },
    { propName: "frozenColumnsAsRowHeaders", type: "boolean", default: false },
    { propName: "frozenRowsAsColumnHeaders", type: "boolean", default: false },
    { propName: "fullRecalc", type: "boolean", default: false },
    { propName: "dynamicReferences", type: "boolean", default: true },
    { propName: "calcOnDemand", type: "boolean", default: false },
    { propName: "includeUnusedStyles", type: "boolean", default: true },
    { propName: "convertSheetTableToDataTable", type: "boolean", default: false },
];

var importSSJsonOptions = [
    { propName: "includeStyles", type: "boolean", default: true },
    { propName: "includeFormulas", type: "boolean", default: true },
    { propName: "frozenColumnsAsRowHeaders", type: "boolean", default: false },
    { propName: "frozenRowsAsColumnHeaders", type: "boolean", default: false },
    { propName: "fullRecalc", type: "boolean", default: false },
    { propName: "incrementalLoading", type: "boolean", default: false }
];

var importCsvOptions = [
    { propName: "encoding", type: "string", default: "UTF-8" },
    { propName: "rowDelimiter", type: "string", default: "\r\n" },
    { propName: "columnDelimiter", type: "string", default: "," }
];

var saveOptions = [
    { propName: "includeBindingSource", type: "boolean", default: false },
    { propName: "includeStyles", type: "boolean", default: true },
    { propName: "includeFormulas", type: "boolean", default: true },
    { propName: "saveAsView", type: "boolean", default: false },
    { propName: "includeAutoMergedCells", type: "boolean", default: false },
    { propName: "includeCalcModelCache", type: "boolean", default: false },
    { propName: "saveR1C1Formula", type: "boolean", default: false },
    { propName: "includeUnusedNames", type: "boolean", default: true },
    { propName: "includeEmptyRegionCells", type: "boolean", default: true },
];

var exportXlsxOptions = [
    { propName: "includeBindingSource", type: "boolean", default: false },
    { propName: "includeStyles", type: "boolean", default: true },
    { propName: "includeFormulas", type: "boolean", default: true },
    { propName: "saveAsView", type: "boolean", default: false },
    { propName: "rowHeadersAsFrozenColumns", type: "boolean", default: false },
    { propName: "columnHeadersAsFrozenRows", type: "boolean", default: false },
    { propName: "includeAutoMergedCells", type: "boolean", default: false },
    { propName: "includeUnusedNames", type: "boolean", default: true },
    { propName: "includeEmptyRegionCells", type: "boolean", default: true },
];

var exportSSJsonOptions = [
    { propName: "includeBindingSource", type: "boolean", default: false },
    { propName: "includeStyles", type: "boolean", default: true },
    { propName: "includeFormulas", type: "boolean", default: true },
    { propName: "saveAsView", type: "boolean", default: false },
    { propName: "rowHeadersAsFrozenColumns", type: "boolean", default: false },
    { propName: "columnHeadersAsFrozenRows", type: "boolean", default: false },
    { propName: "includeAutoMergedCells", type: "boolean", default: false },
];

var exportCsvOptions = [
    { propName: "encoding", type: "string", default: "UTF-8" },
    { propName: "rowDelimiter", type: "string", default: "\r\n" },
    { propName: "columnDelimiter", type: "string", default: "," },
    { propName: "sheetIndex", type: "number", default: 0 },
    { propName: "row", type: "number", default: 0 },
    { propName: "column", type: "number", default: 0 },
    { propName: "rowCount", type: "number", default: 200 },
    { propName: "columnCount", type: "number", default: 20 },
];

var FileType = {
    SJS: 'sjs',
    Excel: 'xlsx',
    SSJson: 'ssjson',
    Csv: 'csv',
}

var openOptionsDict = {};
openOptionsDict[FileType.SJS] = openOptions;
openOptionsDict[FileType.Excel] = importXlsxOptions;
openOptionsDict[FileType.SSJson] = importSSJsonOptions;
openOptionsDict[FileType.Csv] = importCsvOptions;

var saveOptionsDict = {};
saveOptionsDict[FileType.SJS] = saveOptions;
saveOptionsDict[FileType.Excel] = exportXlsxOptions;
saveOptionsDict[FileType.SSJson] = exportSSJsonOptions;
saveOptionsDict[FileType.Csv] = exportCsvOptions;

function initializeSpreadJsSpreadsheet() {

    var spread = new GC.Spread.Sheets.Workbook(document.getElementById("ss"));
    var statusBar = new GC.Spread.Sheets.StatusBar.StatusBar(document.getElementById('statusBar'));
    statusBar.bind(spread);

    $("#import-to-system-btn").kendoButton({

        icon: "import",
        themeColor: "primary",
        click: () => {
            console.log("IN the onClick event handler in import-to-system-btn");
            parseSheetToTsvString();
        }
    });

    $("#kendo-btn-clear").kendoButton({

        icon: "x",
        themeColor: "tertiary",
        click: () => {

            console.log("Clear button onClick event handler:");
            //clearSheetV1();
            clearSheetV2();
            //startSpreadFromScratch();
        }

    });
}

// This is the heart of the initilization code:
function initializeImportExport() {

    var spread = GC.Spread.Sheets.findControl(document.getElementById('ss'));

    initDefaultOptions();

    var selectedFileElement = document.querySelector('#selectedFile');
    selectedFileElement.addEventListener("change", function () {
        var file = selectedFileElement.files[0];
        updateOptions('open', getFileType(file));
    });

    var saveFileType = document.querySelector('#saveFileType');
    saveFileType.addEventListener("change", function () {
        var fileType = saveFileType.value;
        updateOptions('save', fileType);
    });

    document.getElementById('open').onclick = function () {
        var file = document.querySelector('#selectedFile').files[0];
        if (!file) {
            return;
        }

        var fileType = getFileType(file);
        var options = getOptions('open');

        if (fileType === FileType.SJS) {
            spread.open(file, function () { }, function () { }, options);
        } else {
            spread.import(file, function () { }, function () { }, options);
        }
    };
    document.getElementById('save').onclick = function () {
        var fileName = 'export.' + saveFileType.value;
        var fileType = saveFileType.value;
        var options = getOptions('save');

        if (fileType === FileType.SJS) {
            spread.save(function (blob) { saveAs(blob, fileName); }, function () { }, options);
        } else {
            if (fileType === FileType.Csv) {
                var { sheetIndex, row, rowCount, column, columnCount } = options;
                options.range = {
                    sheetIndex: sheetIndex,
                    row: row,
                    rowCount: rowCount,
                    column: column,
                    columnCount: columnCount,
                };
            }
            options.fileType = mapExportFileType(fileType);
            spread.export(function (blob) { saveAs(blob, fileName); }, function () { }, options);
        }
    };
}

//Clear the working spreadsheet:
function startSpreadFromScratch() {

    // // Assuming you have a SpreadJS instance named 'spread'
    try {
        var spreadElement = document.getElementById('ss');
        var spread = GC.Spread.Sheets.findControl(spreadElement);

        // Destroy the existing control if it exists
        if (spread) {
            spread.destroy();
        }

        // Create a new Workbook instance
        spread = new GC.Spread.Sheets.Workbook(spreadElement);

        // Add a default blank sheet
        spread.addSheet(0, new GC.Spread.Sheets.Worksheet('Sheet1'));
    } catch (error) {
        console.error("An error occurred while resetting the SpreadJS control:", error);
    }

}

function clearSheetV2() {
    // Assuming you have a SpreadJS instance named 'spread'
    var spread = GC.Spread.Sheets.findControl(document.getElementById('ss'));

    // Remove all existing sheets
    while (spread.getSheetCount() > 0) {
        spread.removeSheet(0);
    }

    // Add a default blank sheet
    spread.addSheet(0, new GC.Spread.Sheets.Worksheet('Sheet 1'));
}

//Import / Export functionality begins:
// - -  - -- - - - - - - - - - - - - - - - - - - -
function initOptions(options, fileType, mode) {
    var container = document.createElement('div');
    container.className = fileType;
    container.style.display = 'none';

    options.forEach(function (prop) {
        var item = document.createElement('div');
        item.className = 'item';

        var id = getElementId(mode, fileType, prop.propName);
        var label = document.createElement("label");
        label.innerText = prop.displayText || prop.propName;
        label.for = id;

        if (prop.type === 'select') {
            var select = document.createElement('select');
            prop.options.forEach(function (p) {
                var option = document.createElement('option');
                option.value = p.value;
                option.text = p.name;
                select.appendChild(option);
            });

            select.id = id;
            select.value = prop.default;
            item.appendChild(label);
            item.appendChild(select);
        } else {
            var input = document.createElement('input');
            input.id = getElementId(mode, fileType, prop.propName);

            if (prop.type === 'boolean') {
                input.type = 'checkbox';
                input.checked = prop.default;

                item.appendChild(input);
                item.appendChild(label);
            } else if (prop.type === 'number') {
                input.type = 'number';
                input.value = prop.default;

                item.appendChild(label);
                item.appendChild(input);
            } else if (prop.type === 'string') {
                input.type = 'text';
                input.value = prop.default;

                item.appendChild(label);
                item.appendChild(input);
            }
        }

        container.appendChild(item);
    });

    return container;
}

function initDefaultOptions() {
    var container = document.querySelector('.open-options');
    var mode = 'open';

    container.appendChild(initOptions(openOptions, FileType.SJS, mode));
    container.appendChild(initOptions(importXlsxOptions, FileType.Excel, mode));
    container.appendChild(initOptions(importSSJsonOptions, FileType.SSJson, mode));
    container.appendChild(initOptions(importCsvOptions, FileType.Csv, mode));

    container = document.querySelector('.save-options');
    mode = 'save';
    container.appendChild(initOptions(saveOptions, FileType.SJS, mode));
    container.appendChild(initOptions(exportXlsxOptions, FileType.Excel, mode));
    container.appendChild(initOptions(exportSSJsonOptions, FileType.SSJson, mode));
    container.appendChild(initOptions(exportCsvOptions, FileType.Csv, mode));

    updateOptions('save', FileType.SJS);
}

function updateOptions(mode, fileType) {
    var container = document.querySelector('.' + mode + '-options');
    var oldFileType = container.getAttribute('data-file-type');
    if (oldFileType === fileType) {
        return;
    }

    if (oldFileType) {
        container.querySelector('.' + oldFileType).style.display = 'none';
    }
    if (fileType) {
        container.querySelector('.' + fileType).style.display = '';
    }
    container.setAttribute('data-file-type', fileType);
}

function getOptions(mode) {
    var container = document.querySelector('.' + mode + '-options');
    var fileType = container.getAttribute('data-file-type');
    if (!fileType) {
        return;
    }

    var options = {};
    var props = mode === 'open' ? openOptionsDict[fileType] : saveOptionsDict[fileType];
    container = container.querySelector('.' + fileType);

    props.forEach(function (prop) {
        var element = container.querySelector('#' + getElementId(mode, fileType, prop.propName));
        var value;
        if (prop.type === 'boolean') {
            value = element.checked;
        } else if (prop.type === 'number') {
            value = +element.value;
        } else if (prop.type === 'string') {
            value = element.value;
        } else if (prop.type === 'select') {
            value = +element.value;
        }
        options[prop.propName] = value;
    });

    return options;
}

function getElementId(mode, fileType, propName) {
    return mode + '-' + fileType + '-' + propName;
}

function getFileType(file) {
    if (!file) {
        return;
    }

    var fileName = file.name;
    var extensionName = fileName.substring(fileName.lastIndexOf(".") + 1);

    if (extensionName === 'sjs') {
        return FileType.SJS;
    } else if (extensionName === 'xlsx' || extensionName === 'xlsm') {
        return FileType.Excel;
    } else if (extensionName === 'ssjson' || extensionName === 'json') {
        return FileType.SSJson;
    } else if (extensionName === 'csv') {
        return FileType.Csv;
    }
}

function mapExportFileType(fileType) {
    if (fileType === FileType.SSJson) {
        return GC.Spread.Sheets.FileType.ssjson;
    } else if (fileType === FileType.Csv) {
        return GC.Spread.Sheets.FileType.csv;
    }
    return GC.Spread.Sheets.FileType.excel;
}


//Code for parseing the values into TSV:
//
// Assuming you have SpreadJS and the workbook loaded
function parseSheetToTsvString() {
    console.log("in parseSheetToTsvString(): ");
    var spread = GC.Spread.Sheets.findControl(document.getElementById('ss'));
    let sheet = spread.getActiveSheet();
    //let jsonData = spread.toJSON();
    //spread.fromJSON(jsonData, { convertSheetTableToDataTable: false });
    // Generate the TSV string
    let tsvString = createTSVString(sheet);
    console.log(tsvString);
}

// Import the sheet without converting tables to data tables


// Function to create TSV string from sheet data
function createTSVString(sheet) {
    let tsvString = '';
    let rowCount = sheet.getRowCount();
    let colCount = sheet.getColumnCount();

    for (let row = 0; row < rowCount; row++) {
        let rowValues = [];
        for (let col = 0; col < colCount; col++) {
            let value = sheet.getValue(row, col);
            let format = sheet.getFormatter(row, col);

            if (value instanceof Date) {
                console.log("date format?: " + format);
                var dateFormat = validateIsolateFormat(format);
                if (dateFormat) {
                    console.log("date format after split: " + dateFormat + " Actual dateTime value: " + value);
                    //const exactDate = new Date(value.getUTCFullYear(), value.getUTCMonth(), value.getUTCDate(), value.getUTCHours(), value.getUTCMinutes(), value.getUTCSeconds(), value.getUTCMilliseconds());
                    value = formatDate(value, dateFormat);
                } else{
                    alert("the date time format is not valid for this operation.");
                    return;
                }
            }
            rowValues.push(value);
        }

        tsvString += rowValues.join('\t') + '\n';
    }

    return tsvString;
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// dateTime  functions for SpreadJS:
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
const excelToDateFnsFormatMapSpreadJs = {
    'd/m/yy': 'dd/MM/yy',
    'dd-mmm-yy': 'dd-MMM-yy',
    'mm/dd/yyyy': 'MM/dd/yyyy',
    'yyyy-mm-dd': 'yyyy-MM-dd',
    'dd-mmm-yyyy': 'dd-MMM-yyyy',
    'hh:mm:ss': 'HH:mm:ss',
    'dd/mm/yyyy hh:mm': 'dd/MM/yyyy hh:mm',
    'mmm-yy': 'MMM-yy',
    'mmm-yyyy': 'MMM-yyyy',
    'd-mmm': 'd-MMM',
    'd-mmm-yyyy': 'd-MMM-yyyy',
    'yyyy-mm-dd hh:mm:ss': 'yyyy-MM-dd hh:mm:ss',
    'm/d/yy': 'M/d/yy',
    'mm/dd/yy': 'MM/dd/yy',
    'm/d/yy h:mm AM/PM': 'M/d/yy h:mm AM/PM',// M/d/yy h:mm AM/PM
    'mm/dd/yyyy HH:mm:ss': 'MM/dd/yyyy HH:mm:ss',
    'm/d/yy h:mm': 'M/d/yy H:mm',//<--- test case?
    'm/dd/yyyy': 'M/dd/yyyy',
    'mm-dd-yyyy': 'MM-dd-yyyy'
};


function validateIsolateFormat(formatString) {

    //These are all the valid date and time formats:
    const dateFormats = [
        "m/d/yy", "mm/dd/yy", "m/d/yyyy", "mm/dd/yyyy", "d/m/yyyy", "dd/mm/yyyy",
        "yyyy-mm-dd", "dd-mmm-yyyy", "mmm dd, yyyy",
        "m/d/yyyy h:mm AM/PM", "mm/dd/yyyy h:mm AM/PM",
        "d/m/yyyy h:mm AM/PM", "dd/mm/yyyy h:mm AM/PM", 'm/d/yy h:mm', //<-- TEST CASE.
        "yyyy-mm-dd h:mm AM/PM", "dd-mmm-yyyy h:mm AM/PM",
        "mmm dd, yyyy h:mm AM/PM", "h:mm AM/PM", "m/d/yy\ h:mm\ AM/PM"
    ];

    //const simpleDateFormatRegex = /m\/d\/yy\\ h:mm\\ AM\/PM/; //DEBUG.

    //const simpleDateFormatRegex = /m\/d\/yy\\ h:mm/; //DEBUG.

    // Existing regex to match date and time formats
    const dateFormatRegex = /(\b(?:m\/d\/yy\\ h:mm\\ AM\/PM|m\/d\/yy\\ h:mm|m{1,2}\/d{1,2}\/y{2,4} h:mm AM\/PM|d{1,2}\/m{1,2}\/y{2,4} h:mm AM\/PM|yyyy-mm-dd h:mm AM\/PM|dd-mmm-yyyy h:mm AM\/PM|mmm dd, yyyy h:mm AM\/PM|m{1,2}\/d{1,2}\/y{2,4}|d{1,2}\/m{1,2}\/y{2,4}|yyyy-mm-dd|dd-mmm-yyyy|mmm dd, yyyy|m{1,2}\/d{1,2}\/yy|h:mm AM\/PM)\b)/i;

    // Try matching with the simpler regex first
    //let match = formatString.match(simpleDateFormatRegex);

    // If no match, try the existing regex
    //if (!match) {
    match = formatString.match(dateFormatRegex);
    //}

    const cleanedFormat = match ? match[0] : '';

    // Remove backslashes from the cleaned format
    let cleanedFormatNoBackSlash = cleanedFormat.replace(/\\/g, '');

    if (!dateFormats.includes(cleanedFormatNoBackSlash)) {
        return false;
    }

    const cleanedAndConverted = convertExcelFormatToDateFns(cleanedFormatNoBackSlash);

    return cleanedAndConverted;
}

function convertExcelFormatToDateFns(excelFormat) {

    return excelToDateFnsFormatMapSpreadJs[excelFormat] || excelFormat;
}

function formatDate(date, format) {
    var day = date.getDate();
    var month = date.getMonth() + 1; // Months are zero-based
    var year = date.getFullYear();
    var hours = date.getHours();
    var minutes = date.getMinutes();
    var seconds = date.getSeconds();

    var ampm = hours >= 12 ? 'PM' : 'AM';
    var hours12 = hours % 12;
    hours12 = hours12 ? hours12 : 12; // The hour '0' should be '12'

    // Array of month names
    var monthNames = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

    // Temporarily replace month names with placeholders
    format = format.replace("May", "%%May%%").replace("Mar", "%%Mar%%");

    // Create an object with the format tokens and their corresponding values
    var replacements = {
        "yyyy": year,
        "yy": String(year).slice(-2),
        "MMM": monthNames[month - 1],
        "MM": String(month).padStart(2, '0'),
        "M": month,
        "dd": String(day).padStart(2, '0'),
        "d": day,
        "hh": String(hours12).padStart(2, '0'),
        "h": hours12,
        "HH": String(hours).padStart(2, '0'), // 24-hour format
        "H": hours, // 24-hour format
        "mm": String(minutes).padStart(2, '0'),
        "m": minutes,
        "ss": String(seconds).padStart(2, '0'),
        "s": seconds,
        "AM/PM": ampm
    };

    // Replace format tokens with actual values using a regular expression
    format = format.replace(/yyyy|yy|MMM|MM|M|dd|d|hh|h|HH|H|mm|m|ss|s|AM\/PM/g, function (match) {
        return replacements[match];
    });

    // Restore month names from placeholders
    format = format.replace("%%May%%", "May").replace("%%Mar%%", "Mar");

    return format;
}

